@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides support for handling input events.  This provides several classes
 to define input events (InputEvent, KeyboardEvent, BrailleEvent,
@@ -23,13 +23,12 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py 3599 2008-02-19 01:22:55Z joanied $"
-__version__   = "$Revision: 3599 $"
-__date__      = "$Date: 2008-02-18 20:22:55 -0500 (Mon, 18 Feb 2008) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: input_event.py 3971 2008-06-11 00:54:46Z joanied $"
+__version__   = "$Revision: 3971 $"
+__date__      = "$Date: 2008-06-10 20:54:46 -0400 (Tue, 10 Jun 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
 import debug
 import settings
 import time
@@ -70,7 +69,7 @@ class KeyboardEvent(InputEvent):
         # mapping ASCII control characters to UTF-8.]]]
         #
         event_string = event.event_string
-        if (event.modifiers & (1 << pyatspi.MODIFIER_CONTROL)) \
+        if (event.modifiers & settings.CTRL_MODIFIER_MASK) \
             and (not event.is_text) and (len(event_string) == 1):
             value = ord(event.event_string[0])
             if value < 32:
@@ -78,15 +77,8 @@ class KeyboardEvent(InputEvent):
 
         # Filter out the NUMLOCK modifier -- it always causes problems.
         #
-        mask = (1 << settings.MODIFIER_ORCA |
-                1 << pyatspi.MODIFIER_SHIFT |
-                1 << pyatspi.MODIFIER_SHIFTLOCK |
-                1 << pyatspi.MODIFIER_CONTROL |
-                1 << pyatspi.MODIFIER_ALT |
-                1 << pyatspi.MODIFIER_META |
-                1 << pyatspi.MODIFIER_META2 |
-                1 << pyatspi.MODIFIER_META3)
-        event.modifiers = event.modifiers & mask
+        event.modifiers = event.modifiers \
+                          & settings.ALL_BUT_NUMLOCK_MODIFIER_MASK
 
         InputEvent.__init__(self, KEYBOARD_EVENT)
         self.type = event.type
