@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3928 2008-05-27 15:49:33Z richb $"
-__version__   = "$Revision: 3928 $"
-__date__      = "$Date: 2008-05-27 11:49:33 -0400 (Tue, 27 May 2008) $"
+__id__        = "$Id: orca.py 4148 2008-09-03 14:31:54Z wwalker $"
+__version__   = "$Revision: 4148 $"
+__date__      = "$Date: 2008-09-03 10:31:54 -0400 (Wed, 03 Sep 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -34,6 +34,11 @@ __license__   = "LGPL"
 import sys
 sys.argv[0] = "orca"
 
+# Tell Orca to find/use the right version of pygtk.
+#
+import pygtk
+pygtk.require('2.0')
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -60,7 +65,6 @@ import unicodedata
 import pyatspi
 import braille
 import debug
-import dbusserver
 import httpserver
 import keynames
 import keybindings
@@ -75,8 +79,10 @@ from input_event import KeyboardEvent
 from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
+if settings.useDBus:
+    import dbusserver
+
 from orca_i18n import _           # for gettext support
-from orca_i18n import Q_          # to provide qualified translatable strings
 
 if settings.debugMemoryUsage:
     import gc
@@ -183,6 +189,15 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     if obj == orca_state.locusOfFocus:
         return
 
+    # If this event is not for the currently active script, then just return.
+    #
+    if event and event.source and \
+       event.host_application and orca_state.activeScript:
+        currentApp = orca_state.activeScript.app
+        if currentApp != event.host_application and \
+           currentApp != event.source.getApplication():
+            return
+
     oldLocusOfFocus = orca_state.locusOfFocus
     try:
         # Just to see if we have a valid object.
@@ -599,7 +614,9 @@ def keyEcho(event):
 
 def _setClickCount(inputEvent):
     """Sets the count of the number of clicks a user has made to one
-    of the non-modifier keys on the keyboard.
+    of the non-modifier keys on the keyboard.  Note that this looks at
+    the event_string (keysym) instead of hw_code (keycode) because
+    the Java platform gives us completely different keycodes for keys.
 
     Arguments:
     - inputEvent: the current input event.
@@ -613,7 +630,7 @@ def _setClickCount(inputEvent):
         orca_state.clickCount = 0
     elif not isinstance(lastInputEvent, KeyboardEvent):
         orca_state.clickCount = 1
-    elif (lastInputEvent.hw_code != inputEvent.hw_code) or \
+    elif (lastInputEvent.event_string != inputEvent.event_string) or \
          (lastInputEvent.modifiers != inputEvent.modifiers):
         orca_state.clickCount = 1
     elif (inputEvent.time - lastInputEvent.time) < \
@@ -739,7 +756,7 @@ def _processKeyboardEvent(event):
 
     if _orcaModifierPressed:
         keyboardEvent.modifiers = keyboardEvent.modifiers \
-                                  | (1 << settings.MODIFIER_ORCA)
+                                  | settings.ORCA_MODIFIER_MASK
 
     orca_state.lastInputEvent = keyboardEvent
 
@@ -788,6 +805,11 @@ def _processKeyboardEvent(event):
                     exitLearnMode(keyboardEvent)
 
                 consumed = True
+
+            if not consumed \
+               and not isModifierKey(keyboardEvent.event_string) \
+               and keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
+                orca_state.bypassNextCommand = False
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
@@ -864,8 +886,14 @@ def loadUserSettings(script=None, inputEvent=None):
     global _userSettings
 
     # Shutdown the output drivers and give them a chance to die.
+
+    # Only exit the D-Bus server if we're in an environment where there 
+    # is a D-Bus session bus already running.  This helps prevent nastiness
+    # on the login screen.
     #
-    dbusserver.shutdown()
+    if settings.useDBus:
+        dbusserver.shutdown()
+
     httpserver.shutdown()
     speech.shutdown()
     braille.shutdown()
@@ -976,7 +1004,12 @@ def loadUserSettings(script=None, inputEvent=None):
 
     showMainWindowGUI()
 
-    dbusserver.init()
+    # Only start the D-Bus server if we're in an environment where there 
+    # is a D-Bus session bus already running.  This helps prevent nastiness
+    # on the login screen.
+    #
+    if settings.useDBus:
+        dbusserver.init()
     httpserver.init()
 
     return True
@@ -1324,6 +1357,13 @@ def usage():
 
     print "-v, --version                %s" % platform.version
 
+    # Translators: this is a testing option for the command line.  It prints
+    # the names of the applications known to the accessibility infrastructure
+    # to stdout and then exits.
+    #
+    print "-l, --list-apps              " + \
+          _("Print the known running applications")
+
     # Translators: this is the description of the command line option
     # '-s, --setup, --gui-setup' that will initially display a GUI dialog
     # that would allow the user to set their Orca preferences.
@@ -1356,11 +1396,11 @@ def usage():
           _("Use alternate directory for user preferences")
 
     print "-e, --enable=[" \
-        + Q_("option|speech") + "|" \
-        + Q_("option|braille") + "|" \
-        + Q_("option|braille-monitor") + "|" \
-        + Q_("option|magnifier") + "|" \
-        + Q_("option|main-window") + "]",
+        + "speech" + "|" \
+        + "braille" + "|" \
+        + "braille-monitor" + "|" \
+        + "magnifier" + "|" \
+        + "main-window" + "]",
 
     # Translators: if the user supplies an option via the '-e, --enable'
     # command line option, it will be automatically enabled as Orca is
@@ -1369,11 +1409,11 @@ def usage():
     print _("Force use of option")
 
     print "-d, --disable=[" \
-        + Q_("option|speech") + "|" \
-        + Q_("option|braille") + "|" \
-        + Q_("option|braille-monitor") + "|" \
-        + Q_("option|magnifier") + "|" \
-        + Q_("option|main-window") + "]",
+        + "speech" + "|" \
+        + "braille" + "|" \
+        + "braille-monitor" + "|" \
+        + "magnifier" + "|" \
+        + "main-window" + "]",
 
     # Translators: if the user supplies an option via the '-d, --disable'
     # command line option, it will be automatically disabled as Orca is
@@ -1471,7 +1511,7 @@ def main():
         #
         opts, args = getopt.getopt(
             arglist,
-            "?stnvd:e:u:",
+            "?stnvld:e:u:",
             ["help",
              "user-prefs-dir=",
              "enable=",
@@ -1480,6 +1520,7 @@ def main():
              "gui-setup",
              "text-setup",
              "no-setup",
+             "list-apps",
              "version"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
@@ -1493,61 +1534,31 @@ def main():
             if opt in ("-e", "--enable"):
                 feature = val.strip()
 
-                # Translators: this option is for enabling speech synthesis
-                # output.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                if feature == Q_("option|speech"):
+                if feature == "speech":
                     _commandLineSettings["enableSpeech"] = True
-
-                # Translators: this option is for enabling braille output on
-                # a physical refreshable braille display.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|braille"):
+                elif feature == "braille":
                     _commandLineSettings["enableBraille"] = True
-
-                # Translators: this option is for enabling a GUI to monitor
-                # what is being sent to the physical braille display.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|braille-monitor"):
+                elif feature == "braille-monitor":
                     _commandLineSettings["enableBrailleMonitor"] = True
-
-                # Translators: this option is for enabling screen
-                # magnification.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|magnifier"):
+                elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = True
-
-                # Translators: this option is for enabling the display
-                # of the Orca main window.
-                #
-                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                #
-                elif feature == Q_("option|main-window"):
+                elif feature == "main-window":
                     _commandLineSettings["showMainWindow"] = True
-
                 else:
                     usage()
                     abort(2)
 
             if opt in ("-d", "--disable"):
                 feature = val.strip()
-                if feature == Q_("option|speech"):
+                if feature == "speech":
                     _commandLineSettings["enableSpeech"] = False
-                elif feature == Q_("option|braille"):
+                elif feature == "braille":
                     _commandLineSettings["enableBraille"] = False
-                elif feature == Q_("option|braille-monitor"):
+                elif feature == "braille-monitor":
                     _commandLineSettings["enableBrailleMonitor"] = False
-                elif feature == Q_("option|magnifier"):
+                elif feature == "magnifier":
                     _commandLineSettings["enableMagnifier"] = False
-                elif feature == Q_("option|main-window"):
+                elif feature == "main-window":
                     _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
@@ -1567,6 +1578,13 @@ def main():
             if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
                 abort(0)
+            if opt in ("-l", "--list-apps"):
+                apps = filter(lambda x: x is not None,
+                              pyatspi.Registry.getDesktop(0))
+                for app in apps:
+                    print app.name
+                abort(0)
+
     except:
         debug.printException(debug.LEVEL_OFF)
         usage()
