@@ -26,18 +26,22 @@ __date__      = "$Date: $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import gtk
 import pyatspi
 
 import orca.orca as orca
 import orca.debug as debug
 import orca.default as default
+import orca.input_event as input_event
 import orca.orca_state as orca_state
+import orca.settings as settings
 import orca.speech as speech
 import orca.scripts.toolkits.Gecko as Gecko
 
 from orca.orca_i18n import _
 
 from speech_generator import SpeechGenerator
+import script_settings
 
 ########################################################################
 #                                                                      #
@@ -60,6 +64,20 @@ class Script(Gecko.Script):
         # Set the debug level for all the methods in this script.
         self.debugLevel = debug.LEVEL_FINEST
 
+        # Store the last autocompleted string for the address fields
+        # so that we're not too 'chatty'.  See bug #533042.
+        #
+        self._lastAutoComplete = ""
+
+        # When a mail message gets focus, we'll get a window:activate event
+        # followed by two focus events for the document frame.  We want to
+        # present the message if it was just opened; we don't if it was
+        # already opened and the user has just returned focus to it. Store
+        # the fact that a message was loaded which we should present once
+        # the document frame claims focus. See bug #541018.
+        #
+        self._messageLoaded = False
+
         Gecko.Script.__init__(self, app)
 
         # This will be used to cache a handle to the Thunderbird text area for
@@ -72,6 +90,37 @@ class Script(Gecko.Script):
         """
         return SpeechGenerator(self)
 
+    def getAppPreferencesGUI(self):
+        """Return a GtkVBox contain the application unique configuration
+        GUI items for the current application.
+        """
+
+        vbox = Gecko.Script.getAppPreferencesGUI(self)
+
+        # Reapply "say all on load" using the Thunderbird specific setting.
+        #
+        gtk.ToggleButton.set_active(self.sayAllOnLoadCheckButton,
+                                    script_settings.sayAllOnLoad)
+
+        return vbox
+
+    def setAppPreferences(self, prefs):
+        """Write out the application specific preferences lines and set the
+        new values.
+
+        Arguments:
+        - prefs: file handle for application preferences.
+        """
+
+        Gecko.Script.setAppPreferences(self, prefs)
+
+        # Write the Thunderbird specific setting.
+        #
+        prefix = "orca.scripts.apps.Thunderbird.script_settings"
+        value = self.sayAllOnLoadCheckButton.get_active()
+        prefs.writelines("%s.sayAllOnLoad = %s\n" % (prefix, value))
+        script_settings.sayAllOnLoad = value
+
     def _debug(self, msg):
         """ Convenience method for printing debug messages
         """
@@ -119,6 +168,10 @@ class Script(Gecko.Script):
         top = self.getTopLevel(obj)
         consume = False
 
+        # Clear the stored autocomplete string.
+        #
+        self._lastAutoComplete = ""
+
         # Don't speak chrome URLs.
         #
         if obj.name.startswith("chrome://"):
@@ -194,9 +247,87 @@ class Script(Gecko.Script):
         if top and top.getRole() == pyatspi.ROLE_DIALOG:
             self._speakEnclosingPanel(obj)
 
+        # Handle a newly-opened message.
+        #
+        if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME \
+           and orca_state.locusOfFocus.getRole() == pyatspi.ROLE_FRAME:
+            if self._messageLoaded:
+                consume = True
+                self._presentMessage(event.source)
+
+            # If the user just gave focus to the message window (e.g. by
+            # Alt+Tabbing back into it), we might have an existing caret
+            # context. But we'll need the document frame in order to verify
+            # this. Therefore try to find the document frame.
+            #
+            elif self.getCaretContext() == [None, -1]:
+                documentFrame = None
+                for child in orca_state.locusOfFocus:
+                    if child.getRole() == pyatspi.ROLE_INTERNAL_FRAME \
+                       and child.childCount \
+                       and child[0].getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+                        documentFrame = child[0]
+                        break
+                try:
+                    contextObj, contextOffset = \
+                        self._documentFrameCaretContext[hash(documentFrame)]
+                    if contextObj:
+                        orca.setLocusOfFocus(event, contextObj)
+                except:
+                    pass
+
         if not consume:
             Gecko.Script.onFocus(self, event)
 
+    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
+        """Called when the visual object with focus changes.
+
+        Arguments:
+        - event: if not None, the Event that caused the change
+        - oldLocusOfFocus: Accessible that is the old locus of focus
+        - newLocusOfFocus: Accessible that is the new locus of focus
+        """
+
+        # If the user has just deleted a message from the middle of the 
+        # message header list, then we want to speak the newly focused 
+        # message in the header list (even though it has the same row 
+        # number as the previously deleted message).
+        # See bug #536451 for more details.
+        #
+        rolesList = [pyatspi.ROLE_TABLE_CELL, \
+                     pyatspi.ROLE_TREE_TABLE, \
+                     pyatspi.ROLE_SCROLL_PANE, \
+                     pyatspi.ROLE_SCROLL_PANE, \
+                     pyatspi.ROLE_FRAME, \
+                     pyatspi.ROLE_APPLICATION]
+        if self.isDesiredFocusedItem(event.source, rolesList):
+            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+                string = orca_state.lastNonModifierKeyEvent.event_string
+                if string == "Delete":
+                    oldLocusOfFocus = None
+
+        # If the user has just deleted an open mail message, then we want to
+        # try to speak the new name of the open mail message frame.
+        # See bug #540039 for more details.
+        #
+        rolesList = [pyatspi.ROLE_DOCUMENT_FRAME, \
+                     pyatspi.ROLE_INTERNAL_FRAME, \
+                     pyatspi.ROLE_FRAME, \
+                     pyatspi.ROLE_APPLICATION]
+        if self.isDesiredFocusedItem(event.source, rolesList):
+            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+                string = orca_state.lastNonModifierKeyEvent.event_string
+                if string == "Delete":
+                    oldLocusOfFocus = None
+                    state = newLocusOfFocus.getState()
+                    if state.contains(pyatspi.STATE_DEFUNCT):
+                        newLocusOfFocus = event.source
+
+        # Pass the event onto the parent class to be handled in the default way.
+
+        Gecko.Script.locusOfFocusChanged(self, event,
+                                         oldLocusOfFocus, newLocusOfFocus)
+
     def onStateChanged(self, event):
         """Called whenever an object's state changes.
 
@@ -204,11 +335,14 @@ class Script(Gecko.Script):
         - event: the Event
         """
 
-        # For now, we'll bypass the Gecko script's desire to let
-        # someone know when a page has started/completed loading.  The
-        # reason for this is that getting message content from someone
-        # is counted as loading a page.
-        #
+        if event.type.startswith("object:state-changed:busy"):
+            if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME \
+               and not event.detail1:
+                self._messageLoaded = True
+                if self.inDocumentContent():
+                    self._presentMessage(event.source)
+            return
+
         default.Script.onStateChanged(self, event)
 
     def onStateFocused(self, event):
@@ -237,20 +371,24 @@ class Script(Gecko.Script):
         obj = event.source
         parent = obj.parent
 
+        # Speak the autocompleted text, but only if it is different
+        # address so that we're not too "chatty." See bug #533042.
+        #
         if parent.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
-            # Thunderbird does not present all the text in an
-            # autocompletion text entry. This is a workaround.
-            #
-            speech.stop()
+            if event.type.endswith("system") and event.any_data:
+                # The autocompleted address may start with the name,
+                # or it might start with the text typed by the user
+                # followed by ">>" followed by the address. Therefore
+                # we'll look at whatever follows the ">>" should it
+                # exist.
+                #
+                address = event.any_data.split(">>")[-1]
+                if self._lastAutoComplete != address:
+                    speech.speak(address)
+                self._lastAutoComplete = address
+                return
 
-            utterances = []
-            [text, caretOffset, startOffset] = self.getTextLineAtCaret(obj)
-            utterances.append(text)
-            self._debug("onTextInserted: utterances='%s'" % utterances)
-
-            speech.speakUtterances(utterances)
-        else:
-            Gecko.Script.onTextInserted(self, event)
+        Gecko.Script.onTextInserted(self, event)
 
     def onVisibleDataChanged(self, event):
         """Called when the visible data of an object changes."""
@@ -274,6 +412,25 @@ class Script(Gecko.Script):
 
         obj = event.source
 
+        # If the user has just deleted an open mail message, then we want to
+        # try to speak the new name of the open mail message frame and also
+        # present the first line of that message to be consistent with what
+        # we do when a new message window is opened. See bug #540039 for more
+        # details.
+        #
+        rolesList = [pyatspi.ROLE_DOCUMENT_FRAME, \
+                     pyatspi.ROLE_INTERNAL_FRAME, \
+                     pyatspi.ROLE_FRAME, \
+                     pyatspi.ROLE_APPLICATION]
+        if self.isDesiredFocusedItem(event.source, rolesList):
+            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+                string = orca_state.lastNonModifierKeyEvent.event_string
+                if string == "Delete":
+                    speech.speak(obj.name)
+                    [obj, offset] = self.findFirstCaretContext(obj, 0)
+                    self.setCaretPosition(obj, offset)
+                    return
+
         # If we get a "object:property-change:accessible-name" event for 
         # the first item in the Suggestions lists for the spell checking
         # dialog, then speak the first two labels in that dialog. These
@@ -351,13 +508,17 @@ class Script(Gecko.Script):
                     utterances.append(text)
                     speech.speakUtterances(utterances)
 
-    def isLineBreakChar(self, obj, offset):
-        """Returns True of the character at the given offset within
-        obj is a newline.
-        """
+    def _presentMessage(self, documentFrame):
+        """Presents the first line of the message, or the entire message,
+        depending on the user's sayAllOnLoad setting."""
 
-        char = self.getCharacterAtOffset(obj, offset)
-        return char == "\n"
+        [obj, offset] = self.findFirstCaretContext(documentFrame, 0)
+        self.setCaretPosition(obj, offset)
+        if not script_settings.sayAllOnLoad:
+            self.presentLine(obj, offset)
+        elif settings.enableSpeech:
+            self.sayAll(None)
+        self._messageLoaded = False
 
     def getDocumentFrame(self):
         """Returns the document frame that holds the content being shown.
