@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 4109 2008-08-18 16:24:49Z wwalker $"
-__version__   = "$Revision: 4109 $"
-__date__      = "$Date: 2008-08-18 12:24:49 -0400 (Mon, 18 Aug 2008) $"
+__id__        = "$Id: settings.py 4176 2008-09-08 15:50:02Z wwalker $"
+__version__   = "$Revision: 4176 $"
+__date__      = "$Date: 2008-09-08 11:50:02 -0400 (Mon, 08 Sep 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -975,11 +975,6 @@ setScriptMapping(re.compile(_('soffice')), "soffice")
 #
 setScriptMapping(re.compile(_('[Ee]volution')), "evolution")
 
-# Translators: see the regular expression note above.  This is for the
-# help application (yelp).
-#
-setScriptMapping(re.compile(_('yelp')), "Mozilla")
-
 # Translators: see the regular expression note above.  This is for a
 # version of Mozilla Firefox, which chooses to create strange names
 # for itself at the drop of a hat.
@@ -1024,6 +1019,16 @@ setScriptMapping(re.compile(_('vte')), "gnome-terminal")
 #
 setScriptMapping(re.compile(_('gaim')), "pidgin")
 
+# Translators: see the regular expression note above.  This is for
+# supporting yelp, which sometimes identifies itself as gnome-help.
+# [[[TODO - JD: Not marked for translation due to string freeze,
+# plus gnome-help doesn't translate its app name as far as we can
+# determine.  So, translating this string really seems like extra
+# busy work for our translators.]]]
+#
+#setScriptMapping(re.compile(_('gnome-help')), "yelp")
+setScriptMapping(re.compile('gnome-help'), "yelp")
+
 # Show deprecated messeges in debug output.
 # Set this to True to help find potential pyatspi porting problems
 #
